"use strict";
var Promise = require('bluebird');
var notifier = require('node-notifier');
var path = require('path');
var FileSaver_1 = require('./FileSaver');
var ConsoleLogger_1 = require('./../utils/ConsoleLogger');
var OptionsParser_1 = require('./../utils/OptionsParser');
var Defer_1 = require('./../utils/Defer');
Promise.longStackTraces();
var logger = new ConsoleLogger_1.ConsoleLogger();
function spsave(options) {
    return new Promise(function (resolve, reject) {
        var saveOptions = OptionsParser_1.OptionsParser.parseOptions(options);
        var showNotification = function () {
            if (!options.notification) {
                return;
            }
            notifier.notify({
                title: "spsave: " + saveOptions.length + " file(s) uploaded",
                message: saveOptions.map(function (o) { return o.fileName; }).join(', '),
                icon: path.join(__dirname, '../../../assets/sp.png')
            });
        };
        if (saveOptions.length > 1) {
            saveFileArray(saveOptions).then(function (data) {
                showNotification();
                resolve(data);
                return null;
            })
                .catch(function (err) {
                showError(err, options.notification);
                reject(err);
            });
        }
        else if (saveOptions.length === 1) {
            saveSingleFile(saveOptions[0]).then(function (data) {
                showNotification();
                resolve(data);
                return null;
            })
                .catch(function (err) {
                showError(err, options.notification);
                reject(err);
            });
        }
    });
}
exports.spsave = spsave;
function saveFileArray(options, deferred) {
    if (!deferred) {
        deferred = Defer_1.defer();
    }
    if (options.length > 0) {
        saveSingleFile(options[0])
            .then(function () {
            saveFileArray(options.slice(1, options.length), deferred);
            return null;
        })
            .catch(function (err) {
            deferred.reject(err);
        });
    }
    else {
        deferred.resolve(undefined);
    }
    return deferred.promise;
}
function saveSingleFile(options) {
    return new FileSaver_1.FileSaver(options).save();
}
function showError(err, notify) {
    if (notify) {
        notifier.notify({
            title: 'spsave: error occured',
            message: 'For details see console log',
            icon: path.join(__dirname, '../../../assets/sp_error.png')
        });
    }
    if (!err || !err.message) {
        logger.error('Unknown error occured');
        if (err && err.stack) {
            logger.info('');
            logger.info('Stack trace:');
            logger.info('');
            logger.error(err.stack);
        }
        return;
    }
    logger.error('Error occured:');
    if (err.message) {
        logger.error(err.message);
    }
    if (err.stack) {
        logger.info('');
        logger.info('Stack trace:');
        logger.info('');
        logger.error(err.stack);
    }
}
